"""
Tests related to the Config
"""

from django.test import TestCase, override_settings
from django_glide.config import Config


class ConfigTests(TestCase):
    """
    Test case for Config
    """

    def setUp(self):
        self.config = Config()

    def test_js_url(self):
        js_url = self.config.js_url
        self.assertEqual(
            js_url, "https://cdn.jsdelivr.net/npm/@glidejs/glide/dist/glide.min.js"
        )

    @override_settings(GLIDE_JS_URL="my-js-url")
    def test_custom_js_url(self):
        js_url = self.config.js_url
        self.assertEqual(js_url, "my-js-url")

    def test_css_core_url(self):
        css_url = self.config.css_core_url
        self.assertEqual(
            css_url,
            "https://cdn.jsdelivr.net/npm/@glidejs/glide/dist/css/glide.core.min.css",
        )

    @override_settings(GLIDE_CSS_CORE_URL="my-css-url")
    def test_custom_css_core_url(self):
        css_url = self.config.css_core_url
        self.assertEqual(css_url, "my-css-url")

    def test_css_theme_url(self):
        css_url = self.config.css_theme_url
        self.assertEqual(
            css_url,
            "https://cdn.jsdelivr.net/npm/@glidejs/glide/dist/css/glide.theme.min.css",
        )

    @override_settings(GLIDE_CSS_THEME_URL="my-css-url")
    def test_custom_css_core_url(self):
        css_url = self.config.css_theme_url
        self.assertEqual(css_url, "my-css-url")

    @override_settings(GLIDE_CSS_THEME_URL=None)
    def test_none_css_core_url(self):
        css_url = self.config.css_theme_url
        self.assertEqual(css_url, None)

    def test_default_carousel_template(self):
        template = self.config.default_carousel_template
        self.assertEqual(
            template,
            "carousel.html",
        )

    @override_settings(GLIDE_DEFAULT_CAROUSEL_TEMPLATE="test.html")
    def test_custom_default_template(self):
        template = self.config.default_carousel_template
        self.assertEqual(template, "test.html")

    def test_default_slide_template(self):
        template = self.config.default_slide_template
        self.assertEqual(
            template,
            "slide.html",
        )

    @override_settings(GLIDE_DEFAULT_SLIDE_TEMPLATE="test.html")
    def test_custom_default_template(self):
        template = self.config.default_slide_template
        self.assertEqual(template, "test.html")



"""
Contains the config for the library
"""

from django.conf import settings


class Config:
    """
    Configuration taken from Django's settings
    """

    @property
    def js_url(self) -> str:
        """
        Returns the URL to load glide javascript
        """
        return (
            settings.GLIDE_JS_URL
            if hasattr(settings, "GLIDE_JS_URL")
            else "https://cdn.jsdelivr.net/npm/@glidejs/glide/dist/glide.min.js"
        )

    @property
    def css_core_url(self) -> str:
        """
        Returns the URL to load glide core CSS
        """
        return (
            settings.GLIDE_CSS_CORE_URL
            if hasattr(settings, "GLIDE_CSS_CORE_URL")
            else "https://cdn.jsdelivr.net/npm/@glidejs/glide/dist/css/glide.core.min.css"
        )

    @property
    def css_theme_url(self) -> str | None:
        """
        Returns the URL to load glide theme CSS
        It can be None as the theme is optional
        """
        return (
            settings.GLIDE_CSS_THEME_URL
            if hasattr(settings, "GLIDE_CSS_THEME_URL")
            else "https://cdn.jsdelivr.net/npm/@glidejs/glide/dist/css/glide.theme.min.css"
        )

    @property
    def default_carousel_template(self) -> str:
        """
        Returns the default carousel template
        """
        return (
            settings.GLIDE_DEFAULT_CAROUSEL_TEMPLATE
            if hasattr(settings, "GLIDE_DEFAULT_CAROUSEL_TEMPLATE")
            else "carousel.html"
        )

    @property
    def default_slide_template(self) -> str:
        """
        Returns the default slide template
        """
        return (
            settings.GLIDE_DEFAULT_SLIDE_TEMPLATE
            if hasattr(settings, "GLIDE_DEFAULT_SLIDE_TEMPLATE")
            else "slide.html"
        )



"""
Tests related to the template tags
"""

from django.test import TestCase, override_settings
from django.template import Context, Template
from django_glide.config import Config
from django_glide.templatetags.glide_tags import (
    glide_assets,
    normalize_value,
    prepare_options,
)


class TemplateTagsTests(TestCase):
    """
    Test case for the template tags
    """

    def test_js_url(self):
        config = Config()
        expected_data = {
            "js_url": config.js_url,
            "css_core_url": config.css_core_url,
            "css_theme_url": config.css_theme_url,
        }

        self.assertEqual(glide_assets(), expected_data)

    def test_normalize_bool(self):
        expected_value = True
        self.assertEqual(normalize_value("true"), expected_value)

        expected_value = False
        self.assertEqual(normalize_value("false"), expected_value)

    def test_normalize_str(self):
        expected_value = "test"
        self.assertEqual(normalize_value("test"), expected_value)

        expected_value = None
        self.assertEqual(normalize_value("null"), expected_value)

        expected_value = None
        self.assertEqual(normalize_value("none"), expected_value)

    def test_normalize_float(self):
        expected_value = 3.5
        self.assertEqual(normalize_value(3.5), expected_value)
        self.assertEqual(normalize_value("3.5"), expected_value)

    def test_normalize_int(self):
        expected_value = 3
        self.assertEqual(normalize_value(3), expected_value)
        self.assertEqual(normalize_value("3"), expected_value)

    def test_normalize_json(self):
        expected_value = '{1024: {"perView": 4}}'
        self.assertEqual(normalize_value(expected_value), expected_value)

    def test_prepare_options_breakpoints(self):
        options = {"perView": 4, "breakpoints": '{"1024": {"perView": 4}}'}
        expected_value = {"perView": 4, "breakpoints": {"1024": {"perView": 4}}}

        self.assertEqual(prepare_options(**options), expected_value)

    def test_prepare_options_peek(self):
        options = {"perView": 4, "peek": '{"before": 100, "after": 50}'}
        expected_value = {"perView": 4, "peek": {"before": 100, "after": 50}}

        self.assertEqual(prepare_options(**options), expected_value)

    def test_prepare_options_classes(self):
        options = {"perView": 4, "classes": '{"slider": "glide--slider"}'}
        expected_value = {"perView": 4, "classes": {"slider": "glide--slider"}}

        self.assertEqual(prepare_options(**options), expected_value)

    def test_prepare_options_invalid_json(self):
        options = {"perView": 4, "classes": '{"slider": "glide--slider'}
        expected_value = {"perView": 4, "classes": '{"slider": "glide--slider'}

        self.assertEqual(prepare_options(**options), expected_value)



import json
from django import template
from django.template.context import Context
from django.template.loader import get_template
from typing import Dict, List, Any
from django_glide.config import Config

register = template.Library()


def normalize_value(value: Any) -> Any:
    """
    Convert string template args into proper Python types.
    Not the best code, but it will do the trick.
    """
    if isinstance(value, str):
        val = value.strip().lower()
        if val == "true":
            return True
        if val == "false":
            return False
        if val == "null" or val == "none":
            return None
        # try to parse numbers
        try:
            if "." in val:
                return float(val)
            return int(val)
        except ValueError:
            return value  # fallback: leave as string
    return value


def prepare_options(**options: Dict[str, Any]) -> Dict[str, Any]:
    """
    Check for the presence of the breakpoints field to parse it properly. Will throw an exception if invalid.
    """
    for key, value in options.items():
        if key in ("breakpoints", "peek", "classes"):
            try:
                options[key] = json.loads(str(value))
            except (TypeError, json.JSONDecodeError):
                options[key] = normalize_value(value)
        else:
            options[key] = normalize_value(value)

    return options


@register.simple_tag(takes_context=True)
def glide_carousel(
    context: Context,
    items: List[Any],
    carousel_id: str = "glide1",
    carousel_template: str | None = None,
    slide_template: str | None = None,
    **options: Dict[str, Any],
) -> str:
    """
    Render a Glide.js carousel.
    """
    config = Config()

    carousel_template_name = carousel_template or config.default_carousel_template
    slide_template_name = slide_template or config.default_slide_template

    carousel_template = get_template(carousel_template_name)

    ctx = {
        **context.flatten(),
        "items": items,
        "carousel_id": carousel_id,
        "options": json.dumps(prepare_options(**options)),
        "slide_template": slide_template_name,
    }

    return carousel_template.render(ctx)


@register.inclusion_tag("assets.html")
def glide_assets() -> Dict[str, Any]:
    """
    Render Glide.js assets (CSS + JS).
    Should be called once, usually in the <head> or before </body>.
    """
    config = Config()
    return {
        "js_url": config.js_url,
        "css_core_url": config.css_core_url,
        "css_theme_url": config.css_theme_url,
    }
